#include "test.h"

static unsigned char testdata[10 * 1024 * 1024];
static unsigned char testdatacopy[10 * 1024 * 1024];
static unsigned char buff[11 * 1024 * 1024];

/* Check correct behavior on large reads. */
DEFINE_TEST(test_read_large)
{
	int i;
	int tmpfile;
	char tmpfilename[] = "/tmp/test-read_large.XXXXXX";
	size_t used;
	struct archive *a;
	struct archive_entry *entry;

	for (i = 0; i < sizeof(testdata); i++)
		testdata[i] = (unsigned char)(rand());

	assert(NULL != (a = archive_write_new()));
	assertA(0 == archive_write_set_format_ustar(a));
	assertA(0 == archive_write_open_memory(a, buff, sizeof(buff), &used));
	assert(NULL != (entry = archive_entry_new()));
	archive_entry_set_size(entry, sizeof(testdata));
	archive_entry_set_mode(entry, S_IFREG | 0777);
	archive_entry_set_pathname(entry, "test");
	assertA(0 == archive_write_header(a, entry));
	assertA(sizeof(testdata) == archive_write_data(a, testdata, sizeof(testdata)));
#if ARCHIVE_API_VERSION > 1
	assertA(0 == archive_write_finish(a));
#else
	archive_write_finish(a);
#endif

	assert(NULL != (a = archive_read_new()));
	assertA(0 == archive_read_support_format_all(a));
	assertA(0 == archive_read_support_compression_all(a));
	assertA(0 == archive_read_open_memory(a, buff, sizeof(buff)));
	assertA(0 == archive_read_next_header(a, &entry));
	assertA(0 == archive_read_data_into_buffer(a, testdatacopy, sizeof(testdatacopy)));
#if ARCHIVE_API_VERSION > 1
	assertA(0 == archive_read_finish(a));
#else
	archive_read_finish(a);
#endif
	assert(0 == memcmp(testdata, testdatacopy, sizeof(testdata)));


	assert(NULL != (a = archive_read_new()));
	assertA(0 == archive_read_support_format_all(a));
	assertA(0 == archive_read_support_compression_all(a));
	assertA(0 == archive_read_open_memory(a, buff, sizeof(buff)));
	assertA(0 == archive_read_next_header(a, &entry));
	assert(0 < (tmpfile = mkstemp(tmpfilename)));
	assertA(0 == archive_read_data_into_fd(a, tmpfile));
	close(tmpfile);
#if ARCHIVE_API_VERSION > 1
	assertA(0 == archive_read_finish(a));
#else
	archive_read_finish(a);
#endif
	tmpfile = open(tmpfilename, O_RDONLY);
	read(tmpfile, testdatacopy, sizeof(testdatacopy));
	close(tmpfile);
	assert(0 == memcmp(testdata, testdatacopy, sizeof(testdata)));

	unlink(tmpfilename);
}
